@@ -863,6 +863,13 @@ static void storekey(control c,int k)
      p->numkeys++;
 }
 
+static void storetab(control c)
+{
+      ConsoleData p = getdata(c);
+      p->kbuf[(p->firstkey + p->numkeys) % NKEYS] = ' ';
+      p->numkeys++;  
+}
+
 
 #include <Rinternals.h>
 #include <R_ext/Parse.h>
@@ -885,12 +892,16 @@ static void performCompletion(control c)
     SEXP cmdSexp, cmdexpr, ans = R_NilValue;
     ParseStatus status;
 
-    if(!rcompgen_available) return;
+    if(!rcompgen_available) {
+	storetab(c);
+	return;
+    }
     
     if(rcompgen_available < 0) {
 	char *p = getenv("R_COMPLETION");
 	if(p && strcmp(p, "FALSE") == 0) {
 	    rcompgen_available = 0;
+	    storetab(c);
 	    return;	    
 	}
 	/* First check if namespace is loaded */
