@@ -1,6 +1,6 @@
 /*******************************************************************************
  *  BDX: Binary Data eXchange format library
- *  Copyright (C) 1999-2005 Thomas Baier
+ *  Copyright (C) 1999-2006 Thomas Baier
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -13,9 +13,8 @@
  *  Library General Public License for more details.
  * 
  *  You should have received a copy of the GNU Library General Public
- *  License along with this library; if not, write to the Free
- *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *  MA 02110-1301, USA.
+ *  License along with this library; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  *
  *  helper functions for BDX (e.g. memory management)
  *
@@ -51,6 +50,12 @@ struct _BDX_Data* WINAPI bdx_alloc();
 void WINAPI bdx_free(struct _BDX_Data* data);
 void WINAPI bdx_trace(struct _BDX_Data* data); /* trace using OutputDebugString() */
 
+/*
+ * control BDX data conversion
+ */
+void bdx_set_datamode(unsigned long pDM);
+unsigned long bdx_get_datamode();
+
 #ifdef __cplusplus
 }
 #endif
