@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,11 +34,13 @@
 #include <stdio.h>
 #undef DEBUG /* needed for mingw-runtime 2.0 */
 /* the user menu code looks at the internal structure */
+#define GA_EXTERN
 #include "graphapp/internal.h"
 #include "graphapp/ga.h"
 #ifdef USE_MDI
 # include "graphapp/stdimg.h"
 #endif
+
 #include "console.h"
 #include "rui.h"
 #include "preferences.h"
@@ -54,18 +56,18 @@ console RConsole = NULL;
 #ifdef USE_MDI
 int   RguiMDI = RW_MDI | RW_TOOLBAR | RW_STATUSBAR;
 int   MDIset = 0;
-int   R_LoadRconsole = 1;
 window RFrame;
 rect MDIsize;
 #endif
 extern int ConsoleAcceptCmd, R_is_running;
 extern Rboolean DebugMenuitem;
+Rboolean R_LoadRconsole = TRUE; /* used in commandLineArgs */
 
 static menubar RMenuBar;
 static popup RConsolePopup;
 static menuitem msource, mdisplay, mload, msave, mloadhistory,
-    msavehistory, mpaste, mpastecmds, mcopy, mcopypaste, mlazy, mconfig,
-    mls, mrm, msearch, mde, mtools, mstatus;
+    msavehistory, mpaste, mpastecmds, mcopy, mcopypaste, mlazy, mcomplete, 
+    mfncomplete, mconfig, mls, mrm, msearch, mde, mtools, mstatus;
 static int lmanintro, lmanref, lmandata, lmanlang, lmanext, lmanint, lmanadmin;
 static menu m;
 static char cmd[1024];
@@ -348,6 +350,47 @@ static void menulazy(control m)
 /*    show(RConsole); */
 }
 
+extern void set_rcompgen_available(int x);
+
+static int filename_completion_on = 1;
+
+static int check_file_completion(void)
+{
+    /* ought really to ask rcompgen, but that means loading it */
+    return filename_completion_on;
+}
+
+static void menucomplete(control m)
+{
+    if(ischecked(mcomplete)) {
+	set_rcompgen_available(0);
+	uncheck(mcomplete);
+	uncheck(mfncomplete);
+    } else {
+	set_rcompgen_available(-1);
+	check(mcomplete);
+	if(check_file_completion()) check(mfncomplete);
+	else uncheck(mfncomplete);
+    }
+}
+
+static void menufncomplete(control m)
+{
+    char cmd[200], *c0;
+    if(ischecked(mfncomplete)) {
+	c0 = "FALSE";
+	uncheck(mfncomplete);
+	filename_completion_on = 0;
+    } else {
+	c0 = "TRUE";	
+	check(mfncomplete);
+	filename_completion_on = 1;
+    }
+    sprintf(cmd, "rcompgen::rc.settings(files=%s)", c0);
+    consolecmd(RConsole, cmd);
+    
+}
+
 static void menuconsolestayontop(control m)
 {
     BringToTop(RConsole, 2);
@@ -1097,6 +1140,14 @@ int setupui()
 	MCHECK(newmenuitem(G_("Break to debugger"), 0, menudebug));
     MCHECK(newmenuitem("-", 0, NULL));
     MCHECK(mlazy = newmenuitem(G_("Buffered output"), 'W', menulazy));
+    MCHECK(mcomplete = newmenuitem(G_("Word completion"), 0, menucomplete));
+    check(mcomplete);
+    MCHECK(mfncomplete = newmenuitem(G_("Filename completion"), 0, 
+				     menufncomplete));
+    if(check_file_completion())
+	check(mfncomplete);
+    else 
+	uncheck(mfncomplete);
     MCHECK(newmenuitem("-", 0, NULL));
     MCHECK(mls = newmenuitem(G_("List objects"), 0, menuls));
     MCHECK(mrm = newmenuitem(G_("Remove all objects"), 0, menurm));
@@ -1180,7 +1231,7 @@ char *getusermenuname(int pos) {
     return(usermenunames[pos]);
 }
 
-menuItems *wingetmenuitems(char *mname, char *errmsg) {
+menuItems *wingetmenuitems(const char *mname, char *errmsg) {
     menuItems *items;
     char mitem[1002], *p, *q, *r;
     int i,j=0;
@@ -1200,7 +1251,7 @@ menuItems *wingetmenuitems(char *mname, char *errmsg) {
     strcpy(mitem, mname); strcat(mitem, "/");
 
     for (i = 0; i < nitems; i++) {
-	p = (char *)strstr(umitems[i]->name, mitem);
+	p = strstr(umitems[i]->name, mitem);
 
 	if (p == NULL)
 	    continue;
@@ -1248,9 +1299,7 @@ void freemenuitems(menuItems *items) {
     free(items);
 }
 
-extern menu getGraphMenu(char *); /* from devga.c */
-
-static menu getMenu(char * name)
+static menu getMenu(const char * name)
 {
     int i;
     for (i = 0; i < nmenus; i++)
@@ -1264,9 +1313,10 @@ static menu getMenu(char * name)
     else return(NULL);
 }
 
-int winaddmenu(char * name, char *errmsg)
+int winaddmenu(const char *name, char *errmsg)
 {
-    char *p, *submenu = name, start[501];
+    const char *submenu = name;
+    char *p, start[501];
     menu parent;
 
     if (getMenu(name))
@@ -1305,7 +1355,7 @@ int winaddmenu(char * name, char *errmsg)
     }
     if (m) {
 	usermenus[nmenus] = m;
-	usermenunames[nmenus]= strdup(name);
+	usermenunames[nmenus] = strdup(name);
 	nmenus++;
 	show(RConsole);
 	return 0;
@@ -1315,7 +1365,8 @@ int winaddmenu(char * name, char *errmsg)
     }
 }
 
-int winaddmenuitem(char * item, char * menu, char * action, char *errmsg)
+int winaddmenuitem(const char * item, const char * menu, 
+		   const char * action, char *errmsg)
 {
     int i, im;
     menuitem m;
@@ -1396,14 +1447,15 @@ int winaddmenuitem(char * item, char * menu, char * action, char *errmsg)
     return 0;
 }
 
-int windelmenu(char * menu, char *errmsg)
+int windelmenu(const char * menu, char *errmsg)
 {
     int i, j, count = 0, len = strlen(menu);
 
     j = 0;
     for (i = 0; i < nmenus; i++) {
 	if (strcmp(menu, usermenunames[i]) == 0
-	  || (strncmp(menu, usermenunames[i], len) && usermenunames[i][len] == '/')) {
+	  || (strncmp(menu, usermenunames[i], len) == 0 && 
+	      usermenunames[i][len] == '/')) {
 	    remove_menu_item(usermenus[i]);
 	    count++;
 	} else {
@@ -1422,16 +1474,18 @@ int windelmenu(char * menu, char *errmsg)
 
     /* Delete any menu items in this menu */
 
-    for (j = nitems-1; j >= 0; j--) {
-	if (strncmp(menu, umitems[j]->name, len) == 0 && umitems[j]->name[len] == '/')
+    for (j = nitems - 1; j >= 0; j--) {
+	if (strncmp(menu, umitems[j]->name, len) == 0 && 
+	    umitems[j]->name[len] == '/')
 	    windelmenuitem(umitems[j]->name + len + 1, menu, errmsg);
     }
 
+
     show(RConsole);
     return 0;
 }
 
-void windelmenus(char * prefix)
+void windelmenus(const char * prefix)
 {
     int i, len = strlen(prefix);
 
@@ -1441,7 +1495,7 @@ void windelmenus(char * prefix)
     }
 }
 
-int windelmenuitem(char * item, char * menu, char *errmsg)
+int windelmenuitem(const char * item, const char * menu, char *errmsg)
 {
     int i;
     char mitem[1002];
@@ -1464,3 +1518,4 @@ int windelmenuitem(char * item, char * menu, char *errmsg)
     show(RConsole);
     return 0;
 }
+
