@@ -15,8 +15,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,78 +32,85 @@
 #include <ctype.h>
 #include "run.h"
 
-static char RunError[256] = "";
+static char RunError[501] = "";
 
 
-static char * expandcmd(char *cmd)
+static char * expandcmd(const char *cmd)
 {
+    char *buf;
     char  c;
     char *s, *p, *q, *f, *dest, *src, *fl, *fn;
     int   d , ext, len = strlen(cmd)+1;
 
+    /* make a copy as we manipulate in place */
+    buf = alloca(strlen(cmd) + 1);
+    strcpy(buf, cmd);
+
     if (!(s = (char *) malloc(MAX_PATH + strlen(cmd)))) {
 	strcpy(RunError, _("Insufficient memory (expandcmd)"));
 	return NULL;
     }
-    for (p = cmd; *p && isspace(*p); p++);
+    /* skip leading spaces */
+    for (p = buf; *p && isspace(*p); p++);
+    /* find the command itself, possibly double-quoted */
     for (q = p, d = 0; *q && ( d || !isspace(*q) ); q++)
-      if (*q == '\"') d = d ? 0 : 1;
+	if (*q == '\"') d = d ? 0 : 1;
     if (d) {
 	strcpy(RunError, _("A \" is missing (expandcmd)"));
 	return NULL;
     }
-    c = *q;
+    c = *q; /* character after the command, normally a space */
     *q = '\0';
 
 /*
  * I resort to this since SearchPath returned FOUND also
  * for file name without extension -> explicitly set
  *  extension
-*/
-   for ( f = p, ext = 0 ; *f ; f++) {
-     if ((*f == '\\') || (*f == '/')) ext = 0;
-     else if (*f == '.') ext = 1;
-   }
-   /* SearchPath doesn't like \" */
-   fl = alloca(len);
-   fn = alloca(len);
-   for ( dest = fl , src = p; *src ; src++)
-       if (*src != '\"') *dest++ = *src;
-   *dest = '\0';
-   if (ext) {
-   /*
-    * user set extension; we don't check that it is executable;
-    * it might get an error after; but maybe sometimes
-    * in the future every extension will be executable
-   */
-      d = SearchPath(NULL, fl, NULL, MAX_PATH, fn, &f);
-   } else {
-     int iexts = 0;
-     char *exts[] = { ".exe" , ".com" , ".cmd" , ".bat" , NULL };
-     while (exts[iexts]) {
-       strcpy(dest, exts[iexts]);
-       if ((d = SearchPath(NULL, fl, NULL, MAX_PATH, fn, &f))) break;
-       iexts ++ ;
-     }
-   }
-   if (!d) {
-       free(s);
-       strncpy(RunError, p, 200);
-       strcat(RunError, _(" not found"));
-       *q = c;
-       return NULL;
-   }
-   /*
+ */
+    for ( f = p, ext = 0 ; *f ; f++) {
+	if ((*f == '\\') || (*f == '/')) ext = 0;
+	else if (*f == '.') ext = 1;
+    }
+    /* SearchPath doesn't like ", so strip out quotes */
+    fl = alloca(len);
+    fn = alloca(len);
+    for ( dest = fl , src = p; *src ; src++)
+	if (*src != '\"') *dest++ = *src;
+    *dest = '\0';
+    if (ext) {
+	/*
+	 * user set extension; we don't check that it is executable;
+	 * it might get an error after; but maybe sometimes
+	 * in the future every extension will be executable
+	 */
+	d = SearchPath(NULL, fl, NULL, MAX_PATH, fn, &f);
+    } else {
+	int iexts = 0;
+	char *exts[] = { ".exe" , ".com" , ".cmd" , ".bat" , NULL };
+	while (exts[iexts]) {
+	    strcpy(dest, exts[iexts]);
+	    if ((d = SearchPath(NULL, fl, NULL, MAX_PATH, fn, &f))) break;
+	    iexts++ ;
+	}
+    }
+    if (!d) {
+	free(s);
+	strncpy(RunError, p, 200);
+	strcat(RunError, _(" not found"));
+	*q = c;
+	return NULL;
+    }
+    /*
       Paranoia : on my system switching to short names is not needed
       since SearchPath already returns 'short names'. However,
       this is not documented so I prefer to be explicit.
       Problem is that we have removed \" from the executable since
       SearchPath seems dislikes them
-   */
-   GetShortPathName(fn, s, MAX_PATH);
-   *q = c;
-   strcat(s, q);
-   return s;
+    */
+    GetShortPathName(fn, s, MAX_PATH);
+    *q = c;
+    strcat(s, q);
+    return s;
 }
 
 /*
@@ -114,7 +121,7 @@ static char * expandcmd(char *cmd)
    inpipe != 0 to duplicate I/O handles
 */
 
-static HANDLE pcreate(char* cmd, char *finput,
+static HANDLE pcreate(const char* cmd, const char *finput,
 		      int newconsole, int visible, int inpipe)
 {
     DWORD ret;
@@ -129,7 +136,7 @@ static HANDLE pcreate(char* cmd, char *finput,
     sa.lpSecurityDescriptor = NULL;
     sa.bInheritHandle = TRUE;
 
-    if (!(ecmd = expandcmd(cmd)))
+    if (!(ecmd = expandcmd(cmd))) /* error message already set */ 
 	return NULL;
     hTHIS = GetCurrentProcess();
     if (finput && finput[0]) {
@@ -170,8 +177,8 @@ static HANDLE pcreate(char* cmd, char *finput,
 	si.wShowWindow = SW_SHOWDEFAULT;
 	break;
     }
-    ret = CreateProcess(0, (char *) ecmd, &sa, &sa, TRUE,
-		    (newconsole && (visible == 1)) ? CREATE_NEW_CONSOLE : 0,
+    ret = CreateProcess(NULL, ecmd, &sa, &sa, TRUE,
+			(newconsole && (visible == 1)) ? CREATE_NEW_CONSOLE : 0,
 			NULL, NULL, &si, &pi);
     CloseHandle(hTHIS);
     if (finput && finput[0]) {
@@ -228,15 +235,18 @@ char *runerror()
    finput is either NULL or the name of a file from which to
      redirect stdin for the child.
  */
-int runcmd(char *cmd, int wait, int visible, char *finput)
+int runcmd(const char *cmd, int wait, int visible, const char *finput)
 {
     HANDLE p;
     int ret;
 
 /* I hope no program will use this as an error code */
     if (!(p = pcreate(cmd, finput, !wait, visible, 0))) return NOLAUNCH;
-    if (wait) ret = pwait(p);
-    else ret = 0;
+    if (wait) {
+	ret = pwait(p);
+	sprintf(RunError, _("Exit code was %d"), ret);
+	ret &= 0xffff;
+    } else ret = 0;
     CloseHandle(p);
     return ret;
 }
@@ -249,7 +259,7 @@ int runcmd(char *cmd, int wait, int visible, char *finput)
    io = 0 to read stdout from pipe, 1 to write to pipe,
    2 to read stdout and stderr from pipe.
  */
-rpipe * rpipeOpen(char *cmd, int visible, char *finput, int io)
+rpipe * rpipeOpen(const char *cmd, int visible, const char *finput, int io)
 {
     rpipe *r;
     HANDLE hIN, hOUT, hERR, hThread, hTHIS, hTemp;
@@ -384,7 +394,7 @@ int rpipeClose(rpipe * r)
     CloseHandle(r->process);
     i = r->exitcode;
     free(r);
-    return i;
+    return i &= 0xffff;
 }
 
 /* ------------------- Windows pipe connections --------------------- */
@@ -522,9 +532,12 @@ static int Wpipe_vfprintf(Rconnection con, const char *format, va_list ap)
     return res;
 }
 
-Rconnection newWpipe(char *description, char *mode)
+Rconnection newWpipe(const char *description, const char *mode)
 {
     Rconnection new;
+    char *command;
+    int len;
+    
     new = (Rconnection) malloc(sizeof(struct Rconn));
     if(!new) error(_("allocation of pipe connection failed"));
     new->class = (char *) malloc(strlen("pipe") + 1);
@@ -533,12 +546,30 @@ Rconnection newWpipe(char *description, char *mode)
 	error(_("allocation of pipe connection failed"));
     }
     strcpy(new->class, "pipe");
-    new->description = (char *) malloc(strlen(description) + 1);
+    
+    len = strlen(getenv("COMSPEC")) + strlen(description) + 5;
+    command = (char *) malloc(len);
+    if (command) 
+    	new->description = (char *) malloc(len);
+    else
+        new->description = NULL;
+        
     if(!new->description) {
-	free(new->class); free(new);
+    	free(command); free(new->class); free(new);
 	error(_("allocation of pipe connection failed"));
     }
-    init_con(new, description, mode);
+    
+/* We always use COMSPEC here, not R_SHELL or SHELL, for compatibility with Rterm.
+   We also use /c for the same reason.
+*/
+   
+    strcpy(command, getenv("COMSPEC"));
+    strcat(command, " /c ");
+    strcat(command, description);
+    
+    init_con(new, command, mode);
+    free(command);
+    
     new->open = &Wpipe_open;
     new->close = &Wpipe_close;
     new->destroy = &Wpipe_destroy;
@@ -556,3 +587,25 @@ Rconnection newWpipe(char *description, char *mode)
     }
     return new;
 }
+
+
+SEXP do_syswhich(SEXP call, SEXP op, SEXP args, SEXP env)
+{
+    SEXP nm, ans;
+    int i, n;
+
+    checkArity(op, args);
+    nm = CAR(args);
+    if(!isString(nm))
+	error(_("'names' is not a character string"));
+    n = LENGTH(nm);
+    PROTECT(ans = allocVector(STRSXP, n));
+    for(i = 0; i < n; i++) {
+	const char *this = CHAR(STRING_ELT(nm, i)), *that;
+	that = expandcmd(this);
+	SET_STRING_ELT(ans, i, mkChar(that ? that : ""));
+    }
+    setAttrib(ans, R_NamesSymbol, nm);
+    UNPROTECT(1);
+    return ans;
+}
