@@ -116,6 +116,16 @@ int IS_UTF8(SEXP x);
 void SET_UTF8(SEXP x);
 void UNSET_UTF8(SEXP x);
 #endif
+/* macros and declarations for managing CHARSXP cache */
+#define USE_ATTRIB_FIELD_FOR_CHARSXP_CACHE_CHAINS
+#ifdef USE_ATTRIB_FIELD_FOR_CHARSXP_CACHE_CHAINS
+# define CXHEAD(x) (x)
+# define CXTAIL(x) ATTRIB(x)
+SEXP (SET_CXTAIL)(SEXP x, SEXP y);
+#else
+# define CXHEAD(x) CAR(x)
+# define CXTAIL(x) CDR(x)
+#endif /* USE_ATTRIB_FIELD_FOR_CHARSXP_CACHE_CHAINS */
 
 
 #include "Internal.h"		/* do_FOO */
@@ -970,7 +980,7 @@ void jump_to_toplevel(void);
 SEXP levelsgets(SEXP, SEXP);
 void mainloop(void);
 SEXP makeSubscript(SEXP, SEXP, int *, SEXP);
-void markKnown(SEXP, SEXP);
+SEXP markKnown(const char *, SEXP);
 SEXP mat2indsub(SEXP, SEXP, SEXP);
 SEXP matchArg(SEXP, SEXP*);
 SEXP matchArgExact(SEXP, SEXP*);
