@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1998-2001    Robert Gentleman, Ross Ihaka
+ *  Copyright (C) 1998-2007    Robert Gentleman, Ross Ihaka
  *                             and the R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -14,22 +14,24 @@
  *  GNU Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 #ifndef R_EXT_PRINT_H_
 #define R_EXT_PRINT_H_
 
-#include <stdarg.h>
+#ifndef NO_C_HEADERS
+# include <stdarg.h>
+#endif
 
 #ifdef  __cplusplus
 extern "C" {
 #endif
 
-void Rprintf(char*, ...);
-void REprintf(char*, ...);
-void Rvprintf(const char*, va_list);
-void REvprintf(const char*, va_list);
+void Rprintf(char *, ...);
+void REprintf(char *, ...);
+void Rvprintf(const char *, va_list);
+void REvprintf(const char *, va_list);
 
 #ifdef  __cplusplus
 }
