@@ -1,45 +1,63 @@
+#  File src/library/base/R/New-Internal.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 geterrmessage <- function() .Internal(geterrmessage())
 
-try <- function(expr, silent = FALSE)
-{
-    if (! exists("first", inherits = FALSE)) {
-        first <- FALSE
-        # turn on the restart bit of the current context, push an
-        # error handler on the condition handler stack, and push
-        # a tryRestart restart on the restart stack
-        .Internal(.addTryHandlers())
-        if (silent) {
-            op <- options("show.error.messages")
-            on.exit(options(op))
-            options(show.error.messages = FALSE)
+try <- function(expr, silent = FALSE) {
+    tryCatch(expr, error = function(e) {
+        call <- conditionCall(e)
+        if (! is.null(call)) {
+            ## Patch up the call to produce nicer result for testing as
+            ## try(stop(...)).  This will need adjusting if the
+            ## implementation of tryCatch changes.
+            ## Use identical() since call[[1]] can be non-atomic.
+            if (identical(call[[1]], quote(doTryCatch)))
+                call <- sys.call(-4)
+            dcall <- deparse(call)[1]
+            prefix <- paste("Error in", dcall, ": ")
+            LONG <- 75 # to match value in errors.c
+            msg <- conditionMessage(e)
+            sm <- strsplit(msg, "\n")[[1]]
+            if (14 + nchar(dcall, type="w") + nchar(sm[1], type="w") > LONG)
+                prefix <- paste(prefix, "\n  ", sep = "")
         }
-        expr
-    }
-    else invisible(structure(.Internal(geterrmessage()), class = "try-error"))
+        else prefix <- "Error : "
+        msg <- paste(prefix, conditionMessage(e), "\n", sep="")
+        ## Store the error message for legacy uses of try() with
+        ## geterrmessage().
+        .Internal(seterrmessage(msg[1]))
+        if (! silent && identical(getOption("show.error.messages"), TRUE)) {
+            cat(msg, file = stderr())
+            .Internal(printDeferredWarnings())
+        }
+        invisible(structure(msg, class = "try-error"))
+    })
 }
 
-
 comment <- function(x).Internal(comment(x))
 "comment<-" <- function(x,value).Internal("comment<-"(x,value))
 
-round <- function(x, digits = 0).Internal(round(x,digits))
-signif <- function(x, digits = 6).Internal(signif(x,digits))
-logb <- log <- function(x, base=exp(1))
-    if(missing(base)).Internal(log(x)) else .Internal(log(x,base))
-log1p <- function(x).Internal(log1p(x))
-expm1 <- function(x).Internal(expm1(x))
+logb <- function(x, base=exp(1)) if(missing(base)) log(x) else log(x,base)
 
 atan2 <- function(y, x).Internal(atan2(y, x))
 
 beta <- function(a, b).Internal( beta(a, b))
 lbeta <- function(a, b).Internal(lbeta(a, b))
 
-gamma <- function(x).Internal( gamma(x))
-lgamma <- function(x).Internal(lgamma(x))
-digamma <- function(x).Internal(   digamma(x))
-trigamma <- function(x).Internal(  trigamma(x))
-psigamma <- function(x, deriv=0) .Internal(psigamma(x, deriv))
-## tetragamma, pentagamma : deprecated in 1.9.0
+psigamma <- function(x, deriv = 0) .Internal(psigamma(x, deriv))
 
 factorial <- function(x) gamma(x + 1)
 lfactorial <- function(x) lgamma(x + 1)
@@ -48,16 +66,18 @@ choose <- function(n,k).Internal(choose(n,k))
 lchoose <- function(n,k).Internal(lchoose(n,k))
 
 ##-- 2nd part --
-# Machine <- function().Internal(Machine())
 R.Version <- function().Internal(Version())
-commandArgs <- function() .Internal(commandArgs())
+
+commandArgs <- function(trailingOnly = FALSE) {
+    args <- .Internal(commandArgs())
+    if(trailingOnly) {
+        m <- match("--args", args, 0)
+        if(m) args[-(1:m)] else character(0)
+    } else args
+}
 
 args <- function(name).Internal(args(name))
 
-##=== Problems here [[	attr(f, "class") <- "factor"  fails in factor(..)  ]]:
-##- attr <- function(x, which).Internal(attr(x, which))
-##- "attr<-" <- function(x, which, value).Internal("attr<-"(x, which, value))
-
 cbind <- function(..., deparse.level = 1)
     .Internal(cbind(deparse.level, ...))
 
@@ -72,31 +92,29 @@ rbind <- function(..., deparse.level = 1)
 # convert deparsing options to bitmapped integer
 
 .deparseOpts <- function(control) {
-    opts <- pmatch(as.character(control), c("keepInteger", "quoteExpressions",
-      "showAttributes", "useSource", "warnIncomplete", "delayPromises", "all"))
+    opts <- pmatch(as.character(control),
+                   ## the exact order of these is determined by the integer codes in
+                   ## ../../../include/Defn.h
+                   c("all",
+                     "keepInteger", "quoteExpressions", "showAttributes",
+                     "useSource", "warnIncomplete", "delayPromises",
+                     "keepNA", "S_compatible"))
     if (any(is.na(opts)))
         stop(sprintf(ngettext(as.integer(sum(is.na(opts))),
                               "deparse option %s is not recognized",
                               "deparse options %s are not recognized"),
                      paste(sQuote(control[is.na(opts)]), collapse=", ")),
              call. = FALSE, domain = NA)
-    if (any(opts == 7)) {
-	if (length(opts) != 1)
-	    stop("'all' can not be used with other deparse options",
-	       	call. = FALSE)
-	else
-	    return(31)
-    } else return(sum(2^(opts-1)))
+    if (any(opts == 1))
+        opts <- unique(c(opts[opts != 1], 2,3,4,5,6,8))# not (7,9)
+    return(sum(2^(opts-2)))
 }
 
 deparse <-
     function(expr, width.cutoff = 60,
 	     backtick = mode(expr) %in% c("call", "expression", "("),
-	     control = "showAttributes")
-{
-    opts <- .deparseOpts(control)
-    .Internal(deparse(expr, width.cutoff, backtick, opts))
-}
+	     control = c("keepInteger", "showAttributes", "keepNA"))
+    .Internal(deparse(expr, width.cutoff, backtick, .deparseOpts(control)))
 
 do.call <- function(what, args, quote = FALSE, envir = parent.frame())
 {
@@ -116,16 +134,15 @@ format.info <- function(x, digits=NULL, nsmall=0)
 
 gc <- function(verbose = getOption("verbose"),	reset=FALSE)
 {
-    res <- .Internal(gc(verbose,reset))/
-	c(1, 1, 10, 10, 1, 1, rep(10,4), rep(1,2), rep(10,2))
+    res <- .Internal(gc(verbose, reset))
     res <- matrix(res, 2, 7,
 		  dimnames = list(c("Ncells","Vcells"),
 		  c("used", "(Mb)", "gc trigger", "(Mb)",
 		    "limit (Mb)", "max used", "(Mb)")))
     if(all(is.na(res[, 5]))) res[, -5] else res
 }
-gcinfo <- function(verbose).Internal(gcinfo(verbose))
-gctorture <- function(on=TRUE)invisible(.Internal(gctorture(on)))
+gcinfo <- function(verbose) .Internal(gcinfo(verbose))
+gctorture <- function(on=TRUE) invisible(.Internal(gctorture(on)))
 
 is.unsorted <- function(x, na.rm = FALSE) {
     if(is.null(x)) return(FALSE)
@@ -139,13 +156,11 @@ is.unsorted <- function(x, na.rm = FALSE) {
 }
 
 mem.limits <- function(nsize=NA, vsize=NA)
-{
     structure(.Internal(mem.limits(as.integer(nsize), as.integer(vsize))),
               names=c("nsize", "vsize"))
-}
 
-nchar <- function(x, type = "bytes")
-    .Internal(nchar(x, type))
+nchar <- function(x, type = "chars", allowNA = FALSE)
+    .Internal(nchar(x, type, allowNA))
 
 polyroot <- function(z).Internal(polyroot(z))
 
@@ -199,8 +214,8 @@ data.class <- function(x) {
     }
 }
 
-is.numeric.factor <- function(x) FALSE
-is.integer.factor <- function(x) FALSE
+## is.numeric.factor <- function(x) FALSE # internal in 2.5.0
+## is.integer.factor <- function(x) FALSE # internal in 2.5.0
 
 encodeString <- function(x, width = 0, quote = "", na.encode = TRUE,
                          justify = c("left", "right", "centre", "none"))
@@ -216,7 +231,7 @@ encodeString <- function(x, width = 0, quote = "", na.encode = TRUE,
 
 l10n_info <- function() .Internal(l10n_info())
 
-iconv <- function(x, from, to, sub = NA) {
+iconv <- function(x, from = "", to = "", sub = NA) {
     if(!is.character(x)) x <- as.character(x)
     .Internal(iconv(x, from, to, as.character(sub)))
 }
@@ -226,7 +241,8 @@ iconvlist <- function()
     int <- .Internal(iconv(NULL, "", "", ""))
     if(length(int)) return(sort.int(int))
     icfile <- system.file("iconvlist", package="utils")
-    if(!nchar(icfile)) stop("'iconvlist' is not available on this system")
+    if(!nchar(icfile, type="bytes"))
+        stop("'iconvlist' is not available on this system")
     ext <- readLines(icfile)
     if(!length(ext)) stop("'iconvlist' is not available on this system")
     ## glibc has lines ending //, some versions with a header and some without.
@@ -244,5 +260,8 @@ Cstack_info <- function() .Internal(Cstack_info())
 reg.finalizer <- function(e, f, onexit = FALSE)
     .Internal(reg.finalizer(e, f, onexit))
 
+Encoding <- function(x) .Internal(Encoding(x))
+`Encoding<-` <- function(x, value) .Internal(setEncoding(x, value))
+
 ## base has no S4 generics
 .noGenerics <- TRUE
