@@ -1,3 +1,19 @@
+#  File src/library/base/R/connections.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 stdin <- function() .Internal(stdin())
 stdout <- function() .Internal(stdout())
 stderr <- function() .Internal(stderr())
@@ -60,7 +76,7 @@ file <- function(description = "", open = "", blocking = TRUE,
 pipe <- function(description, open = "", encoding = getOption("encoding"))
     .Internal(pipe(description, open, encoding))
 
-fifo <- function(description = "", open = "", blocking = FALSE,
+fifo <- function(description, open = "", blocking = FALSE,
                  encoding = getOption("encoding"))
     .Internal(fifo(description, open, blocking, encoding))
 
@@ -143,12 +159,7 @@ showConnections <- function(all = FALSE)
 getAllConnections <- function()
     .Internal(getAllConnections())
 
-getConnection <- function(what)
-{
-    set <- getAllConnections()
-    if(what %in% set) structure(what, class="connection")
-    else NULL
-}
+getConnection <- function(what) .Internal(getConnection(what))
 
 closeAllConnections <- function()
 {
