@@ -65,6 +65,8 @@ rownames <- function(x, do.NULL = TRUE, prefix = "row")
 
 colnames <- function(x, do.NULL = TRUE, prefix = "col")
 {
+    if(is.data.frame(x) && do.NULL)
+	return(names(x))
     dn <- dimnames(x)
     if(!is.null(dn[[2]]))
 	dn[[2]]
