@@ -1,6 +1,22 @@
+#  File src/library/methods/R/methodsTable.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 ### merge version called from namespace imports code.  Hope to avoid using generic
 .mergeMethodsTable2 <- function(table, newtable, envir, metaname) {
-    old <- objects(table, all=TRUE)
+    old <- objects(table, all.names=TRUE)
     mm <- 1
     for( what in old) {
       mm = get(what, envir =table)
@@ -9,7 +25,7 @@
           break
       }
     }
-    new = objects(newtable, all=TRUE)
+    new = objects(newtable, all.names=TRUE)
     ## check that signature length doesn't change
     canStore <- TRUE
     for(what in new) {
@@ -43,7 +59,7 @@
   n <- get(".SigLength", envir = fenv)
   anySig <- rep("ANY", n)
   anyLabel <- .sigLabel(anySig)
-  newMethods <- objects(newtable, all=TRUE)
+  newMethods <- objects(newtable, all.names=TRUE)
   for(what in newMethods) {
     obj <- get(what, envir = newtable)
     if(is.primitive(obj))
@@ -53,7 +69,7 @@
     else
       stop(gettextf(
                     "Invalid object in meta table of methods for \"%s\", label \"%s\", had class \"%s\"",
-                    generic@generic, what, class(obj)))
+                    generic@generic, what, class(obj)), domain=NA)
     ns <- length(sig)
     if(ns == n) {}
     else {
@@ -287,7 +303,8 @@
 	warning(gettextf(paste("Ambiguous method selection for \"%s\", target \"%s\"",
                                "(the first of the signatures shown will be used)\n%s\n"),
 			 fdef@generic, .sigLabel(classes),
-			 paste("   ", names(methods), collapse = "\n")))
+			 paste("   ", names(methods), collapse = "\n")),
+                domain = NA, call. = FALSE)
       methods <- methods[1]
   }
   if(doMtable  && length(methods)>0) {
@@ -457,7 +474,7 @@
     else {
       warning(gettextf(
               "Couldn't find methods table for \"%s\", package \"%s\" may be out of date",
-                       generic@generic, generic@package))
+                       generic@generic, generic@package), domain=NA)
       metaName <- mlistMetaName(generic)
       if(exists(metaName, envir = env, inherits = FALSE))
         .mlistAddToTable(generic, get(metaName, envir = env), mtable, attach)
@@ -481,12 +498,12 @@
 
 .resetInheritedMethods <- function(fenv, mtable) {
     allObjects <- character()
-    direct <- objects(mtable, all=TRUE)
+    direct <- objects(mtable, all.names=TRUE)
     if(exists(".AllMTable", envir = fenv, inherits = FALSE)) {
         ## remove all inherited methods.  Note that code (e.g. setMethod) that asigns
         ## a new method to mtable is responsible for copying it to allTable as well.
         allTable <- get(".AllMTable", envir = fenv)
-        allObjects <- objects(allTable, all=TRUE)
+        allObjects <- objects(allTable, all.names=TRUE)
         remove(list= allObjects[is.na(match(allObjects, direct))], envir = allTable)
     }
     else {
@@ -495,7 +512,7 @@
     }
     ## check for missing direct objects; usually a non-existent AllMTable?
     if(any(is.na(match(direct, allObjects)))) {
-        direct <- objects(mtable, all=TRUE)
+        direct <- objects(mtable, all.names=TRUE)
         for(what in direct)
           assign(what, get(what, envir = mtable), envir = allTable)
     }
@@ -511,7 +528,8 @@
 	if(!is.na(onOff))
 	    .assignOverBinding(".UseMethodsTables", onOff, where = where, FALSE)
 	else
-	    stop(gettextf(".UsingMethodsTables: 'onOff' is not TRUE or FALSE"))
+	    stop(gettextf(".UsingMethodsTables: 'onOff' is not TRUE or FALSE"),
+                 domain = NA)
     }
     prev
 }
@@ -528,13 +546,13 @@
     qs <- function(what) paste('"', what, '"', collapse = ", ", sep = "")
     doFun <- function(func, pkg) cf("Function: ", func, " (package ", pkg, ")\n")
     env <- environment(generic)
-    signature = generic@signature
+    signature <- generic@signature
     table <- get(if(inherited) ".AllMTable" else ".MTable", envir = env)
     f <- generic@generic
     p <- packageSlot(f)
     if(is.null(p)) p <- "base"
     deflt <- new("signature", generic, "ANY")
-    labels <- objects(table, all = TRUE)
+    labels <- objects(table, all.names = TRUE)
     if(!is.null(classes) && length(labels) > 0) {
 	sigL <- strsplit(labels, split = "#")
 	keep <- !sapply(sigL, function(x, y) all(is.na(match(x, y))), classes)
@@ -561,7 +579,8 @@
 	    cf(sigString(t), "\n")
 	    if(!identical(t, d))
 		cf("    (inherited from: ", sigString(d), ")\n")
-            if(!.identC(m@generic, f) && length(m@generic) == 1 && nchar(m@generic)>0)
+            if(!.identC(m@generic, f) && length(m@generic) == 1 &&
+               nzchar(m@generic))
 		cf("    (definition from function \"", m@generic, "\")\n")
 	}
 	if(includeDefs) {
@@ -776,7 +795,7 @@ outerLabels <- function(labels, new) {
     if(is.null(generic)) {
       warning(gettextf(
        "Could not find generic function \"%s\" to initialize cached methods",
-                       name))
+                       name), domain=NA)
       next
     }
     table <- .getMethodsTable(generic)
@@ -793,6 +812,10 @@ outerLabels <- function(labels, new) {
     }
 }
 
+.removeMethodsMetaTable <- function(generic, where) {
+    rm(list=.TableMetaName(generic@generic, generic@package), pos = where)
+}
+
 .getGenericSigArgs <- function(fdef, env = environment(fdef), check = TRUE) {
     if(check && !exists(".SigLength", envir = env, inherits = FALSE))
       .setupMethodsTables(fdef)
@@ -803,6 +826,17 @@ outerLabels <- function(labels, new) {
 }
 
 
+## the most simple part of listFromMethods() below; not yet exported
+tableNames <- function(generic, where, table) {
+    fdef <- getGeneric(generic)
+    if(missing(table))
+	table <-
+	    if(missing(where)) .getMethodsTable(fdef)
+	    else get(.TableMetaName(fdef@generic, fdef@package),
+                     envir = as.environment(where), inherits = FALSE)
+    objects(table, all.names=TRUE)
+}
+
 listFromMethods <- function(generic, where, table) {
     fdef <- getGeneric(generic)
     if(missing(table))
@@ -812,7 +846,7 @@ listFromMethods <- function(generic, where, table) {
 		     envir = as.environment(where), inherits = FALSE)
     fev <- environment(fdef)
     nSigArgs <- .getGenericSigLength(fdef, fev)
-    names <- objects(table, all=TRUE)
+    names <- objects(table, all.names=TRUE)
     methods <- lapply(names, function(x)get(x, envir = table))
     if(nSigArgs > 1) {
         n <- length(names)
