@@ -26,21 +26,20 @@ confint.lm <- function(object, parm, level = 0.95, ...)
 {
     cf <- coef(object)
     pnames <- names(cf)
-    if(missing(parm))
-	parm <- seq_along(pnames)
-    else if(is.character(parm))
-	parm <- match(parm, pnames, nomatch = 0)
+    if(missing(parm)) parm <- pnames
+    else if(is.numeric(parm)) parm <- pnames[parm]
     a <- (1 - level)/2
     a <- c(a, 1 - a)
+    fac <- qt(a, object$df.residual) # difference from default method
     pct <- format.perc(a, 3)
     ci <- array(NA, dim = c(length(parm), 2),
-		dimnames = list(pnames[parm], pct))
-    ses <- sqrt(diag(vcov(object)))[parm]
-    fac <- qt(a, object$df.residual)
+		dimnames = list(parm, pct))
+    ses <- sqrt(diag(vcov(object)))[parm] # gives NA for aliased parms
     ci[] <- cf[parm] + ses %o% fac
     ci
 }
 
+
 confint.glm <- function(object, parm, level = 0.95, ...)
     try(MASS:::confint.glm(object, parm, level, ...))
 
@@ -51,16 +50,14 @@ confint.default <- function (object, parm, level = 0.95, ...)
 {
     cf <- coef(object)
     pnames <- names(cf)
-    if (missing(parm))
-	parm <- seq_along(pnames)
-    else if (is.character(parm))
-	parm <- match(parm, pnames, nomatch = 0)
+    if(missing(parm)) parm <- pnames
+    else if(is.numeric(parm)) parm <- pnames[parm]
     a <- (1 - level)/2
     a <- c(a, 1 - a)
     pct <- format.perc(a, 3)
     fac <- qnorm(a)
     ci <- array(NA, dim = c(length(parm), 2),
-		dimnames = list(pnames[parm], pct))
+		dimnames = list(parm, pct))
     ses <- sqrt(diag(vcov(object)))[parm]
     ci[] <- cf[parm] + ses %o% fac
     ci
