@@ -1,3 +1,19 @@
+#  File src/library/stats/R/loess.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 loess <-
 function(formula, data, weights, subset, na.action, model = FALSE,
 	 span = 0.75, enp.target, degree = 2, parametric = FALSE,
@@ -350,7 +366,7 @@ predLoess <-
 pointwise <- function(results, coverage)
 {
     fit <- results$fit
-    lim <- qt((1 - coverage)/2, results$df, lower = FALSE) * results$se.fit
+    lim <- qt((1 - coverage)/2, results$df, lower.tail = FALSE) * results$se.fit
     list(fit = fit, lower = fit - lim, upper = fit + lim)
 }
 
@@ -425,7 +441,7 @@ loess.smooth <-
 	   evaluation = 50, ...)
 {
     notna <- !(is.na(x) | is.na(y))
-    new.x <- seq(min(x[notna]), max(x[notna]), length = evaluation)
+    new.x <- seq.int(min(x[notna]), max(x[notna]), length.out = evaluation)
 
     control <- loess.control(...)
     ##	x <- matrix(x, ncol = 1)
