@@ -1,3 +1,19 @@
+#  File src/library/stats/R/manova.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 manova <- function(...)
 {
     Call <- fcall <- match.call()
@@ -32,7 +48,7 @@ summary.manova <-
     effects <- object$effects
     if (!is.null(effects))
         effects <- as.matrix(effects)[seq_along(asgn), , drop = FALSE]
-    rdf <- object$df.resid
+    rdf <- object$df.residual
     nmeffect <- c("(Intercept)", attr(object$terms, "term.labels"))
     resid <- as.matrix(object$residuals)
     wt <- object$weights
