@@ -145,13 +145,13 @@ C
 C  MCQUITTY'S METHOD - IOPT=5.
 C
          IF (IOPT.EQ.5) THEN
-            DISS(IND1)=0.5*DISS(IND1)+0.5*DISS(IND2)
+            DISS(IND1)=0.5D0*DISS(IND1)+0.5D0*DISS(IND2)
          ENDIF
 C
 C  MEDIAN (GOWER'S) METHOD - IOPT=6.
 C
          IF (IOPT.EQ.6) THEN
-            DISS(IND1)=0.5*DISS(IND1)+0.5*DISS(IND2)-0.25*D12
+            DISS(IND1)=0.5D0*DISS(IND1)+0.5D0*DISS(IND2)-0.25D0*D12
          ENDIF
 C
 C  CENTROID METHOD - IOPT=7.
