@@ -1,4 +1,21 @@
-dataentry <- function (data, modes) {
+#  File src/library/utils/R/edit.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
+dataentry <- function (data, modes)
+{
     if(!is.list(data) || !length(data) || !all(sapply(data, is.vector)))
         stop("invalid 'data' argument")
     if(!is.list(modes) ||
@@ -7,6 +24,25 @@ dataentry <- function (data, modes) {
     .Internal(dataentry(data, modes))
 }
 
+View <- function (x, title)
+{
+    if(missing(title)) title <- paste("Data:", deparse(substitute(x)))
+    as.num.or.char <- function(x)
+    {
+        if (is.character(x)) x
+        else if(is.numeric(x)) {storage.mode(x) <- "double"; x}
+        else as.character(x)
+    }
+    x0 <- as.data.frame(x)
+    x <- lapply(x0, as.num.or.char)
+    rn <- row.names(x0)
+    if(any(rn != seq_along(rn))) x <- c(list(row.names = rn), x)
+    if(!is.list(x) || !length(x) || !all(sapply(x, is.atomic)) ||
+       !max(sapply(x, length)))
+        stop("invalid 'x' argument")
+    .Internal(dataviewer(x, title))
+}
+
 edit <- function(name,...)UseMethod("edit")
 
 edit.default <-
@@ -72,7 +108,7 @@ edit.data.frame <-
         out <- out[-1]
         if((ln <- length(rn)) < maxlength)
             rn <- c(rn, paste("row", (ln+1):maxlength, sep=""))
-    } else if(length(rn) != maxlength) rn <- seq(len=maxlength)
+    } else if(length(rn) != maxlength) rn <- seq_len(maxlength)
     for (i in factors) {
         if(factor.mode != mode(out[[i]])) next # user might have switched mode
         a <- attrlist[[i]]
@@ -106,7 +142,7 @@ edit.data.frame <-
     if (edit.row.names) {
         if(any(duplicated(rn))) {
             warning("edited row names contain duplicates and will be ignored")
-            attr(out, "row.names") <- seq(len=maxlength)
+            attr(out, "row.names") <- seq_len(maxlength)
         }
     }
     out
