@@ -1,3 +1,19 @@
+#  File src/library/utils/R/objects.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 ## internal function used only in this file
 findGeneric <- function(fname, envir)
 {
@@ -13,14 +29,14 @@ findGeneric <- function(fname, envir)
     }
     isUMEbrace <- function(e) {
         for (ee in as.list(e[-1]))
-            if (nchar(res <- isUME(ee))) return(res)
+            if (nzchar(res <- isUME(ee))) return(res)
         ""
     }
     isUMEif <- function(e) {
         if (length(e) == 3) isUME(e[[3]])
         else {
-            if (nchar(res <- isUME(e[[3]]))) res
-            else if (nchar(res <- isUME(e[[4]]))) res
+            if (nzchar(res <- isUME(e[[3]]))) res
+            else if (nzchar(res <- isUME(e[[4]]))) res
             else ""
         }
     }
@@ -56,7 +72,8 @@ methods <- function (generic.function, class)
     knownGenerics <- c(names(.knownS3Generics),
                        tools:::.get_internal_S3_generics())
 
-    an <- lapply(seq(along=(sp <- search())), ls)
+    sp <- search()
+    an <- lapply(seq_along(sp), ls)
     names(an) <- sp
     an <- unlist(an)
     an <- an[!duplicated(an)] # removed masked objects, *keep* names
@@ -68,7 +85,8 @@ methods <- function (generic.function, class)
 	if (!is.character(generic.function))
 	    generic.function <- deparse(substitute(generic.function))
         else if(!exists(generic.function, mode = "function",
-                        envir = parent.frame()))
+                        envir = parent.frame()) &&
+                !generic.function %in% c("Math", "Ops", "Complex", "Summary"))
             stop(gettextf("no function '%s' is visible", generic.function),
                  domain = NA)
         if(!any(generic.function == knownGenerics)) {
@@ -76,7 +94,7 @@ methods <- function (generic.function, class)
             if(truegf == "")
                 warning(gettextf("function '%s' appears not to be generic",
                                  generic.function), domain = NA)
-            if(nchar(truegf) && truegf != generic.function) {
+            if(nzchar(truegf) && truegf != generic.function) {
                 warning(gettextf("generic function '%s' dispatches methods for generic '%s'",
                         generic.function, truegf), domain = NA)
                 generic.function <- truegf
@@ -132,7 +150,7 @@ methods <- function (generic.function, class)
                 where <- find(nm, mode = "function")
                 if(!length(where)) return(FALSE)
                 any(sapply(where, function(w)
-                           nchar(findGeneric(nm, envir=as.environment(w))) > 0))
+                           nzchar(findGeneric(nm, envir=as.environment(w)))))
             })
             info <- info[keep, ]
         }
@@ -178,7 +196,7 @@ getS3method <-  function(f, class, optional = FALSE)
                        names(.knownS3Generics))
     if(!any(f == knownGenerics)) {
         truegf <- findGeneric(f, parent.frame())
-        if(nchar(truegf)) f <- truegf
+        if(nzchar(truegf)) f <- truegf
         else {
             if(optional) return(NULL)
             else stop(gettextf("no function '%s' could be found", f), domain = NA)
@@ -198,8 +216,7 @@ getS3method <-  function(f, class, optional = FALSE)
         else .BaseNamespaceEnv
     }
     S3Table <- get(".__S3MethodsTable__.", envir = defenv)
-    S3reg <- ls(S3Table)
-    if(length(grep(gsub("([.[$])", "\\\\\\1", method), S3reg)))
+    if(exists(method, envir = S3Table, inherits = FALSE))
         return(get(method, envir = S3Table))
     if(optional) NULL else stop(gettextf("S3 method '%s' not found", method),
                                 domain = NA)
@@ -208,7 +225,7 @@ getS3method <-  function(f, class, optional = FALSE)
 getFromNamespace <- function(x, ns, pos = -1, envir = as.environment(pos))
 {
     if(missing(ns)) {
-        nm <- attr(envir, "name")
+        nm <- attr(envir, "name", exact = TRUE)
         if(is.null(nm) || substring(nm, 1, 8) != "package:")
             stop("environment specified is not a package")
         ns <- asNamespace(substring(nm, 9))
@@ -220,7 +237,7 @@ assignInNamespace <-
     function(x, value, ns, pos = -1, envir = as.environment(pos))
 {
     if(missing(ns)) {
-        nm <- attr(envir, "name")
+        nm <- attr(envir, "name", exact = TRUE)
         if(is.null(nm) || substring(nm, 1, 8) != "package:")
             stop("environment specified is not a package")
         ns <- asNamespace(substring(nm, 9))
@@ -264,7 +281,7 @@ fixInNamespace <- function (x, ns, pos = -1, envir = as.environment(pos), ...)
     if (!is.character(subx) || length(subx) != 1)
         stop("'fixInNamespace' requires a name")
     if(missing(ns)) {
-        nm <- attr(envir, "name")
+        nm <- attr(envir, "name", exact = TRUE)
         if(is.null(nm) || substring(nm, 1, 8) != "package:")
             stop("environment specified is not a package")
         ns <- asNamespace(substring(nm, 9))
@@ -278,7 +295,7 @@ getAnywhere <- function(x)
     x <- as.character(substitute(x))
     objs <- list(); where <- character(0); visible <- logical(0)
     ## first look on search path
-    if(length(pos <- find(x, numeric=TRUE))) {
+    if(length(pos <- find(x, numeric = TRUE))) {
         objs <- lapply(pos, function(pos, x) get(x, pos=pos), x=x)
         where <- names(pos)
         visible <- rep.int(TRUE, length(pos))
@@ -358,3 +375,14 @@ print.getAnywhere <- function(x, ...)
     if(length(i) == 1) x$objs[[i]]
     else x$objs[i]
 }
+
+argsAnywhere <- function(x) {
+        name<-as.character(substitute(x))
+        fs<-do.call(getAnywhere, list(name))
+        if (sum(!fs$dups)==0)
+            return(NULL)
+        if (sum(!fs$dups)>1)
+            sapply(fs$objs[!fs$dups],
+                   function(f) if (is.function(f)) args(f))
+        else args(fs$objs[[1]])
+}
