@@ -1,3 +1,19 @@
+#  File src/library/utils/R/windows/help.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 .show_help_on_topic_as_HTML <-
 function(file, topic)
 {
@@ -17,7 +33,7 @@ function(file, topic)
         "paper]{article}",
         "\n",
         "\\usepackage[",
-        if(nchar(opt <- Sys.getenv("R_RD4DVI"))) opt else "ae",
+        if(nzchar(opt <- Sys.getenv("R_RD4DVI"))) opt else "ae",
         "]{Rd}",
         "\n",
         "\\InputIfFileExists{Rhelp.cfg}{}{}\n",
@@ -26,10 +42,10 @@ function(file, topic)
     file.append(con, file)
     cat("\\end{document}\n", file = con, append = TRUE)
     cmd <- paste('"',
-                 paste(R.home(), "bin", "helpPRINT", sep="/"),
+                 paste(R.home("bin"), "helpPRINT", sep="/"),
                  '"', sep="")
     texpath <- chartr("\\", "/",
-                      file.path(R.home(), "share", "texmf"))
+                      file.path(R.home("share"), "texmf"))
     system(paste(cmd, FILE, topic, texpath), wait = FALSE)
     return(invisible())
 }
