@@ -1,22 +1,21 @@
 /*
-  R : A Computer Language for Statistical Data Analysis
-  Copyright (C) 1997-2006   Robert Gentleman, Ross Ihaka
-                            and the R Development Core Team
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or (at
-  your option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-  U.S.A.
+ *   R : A Computer Language for Statistical Data Analysis
+ *   Copyright (C) 1997-2007   Robert Gentleman, Ross Ihaka
+ *                             and the R Development Core Team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 /* <UTF8> This does byte-level access, e.g. isspace, but is OK. */
@@ -29,14 +28,10 @@
 #include <config.h>
 #endif
 
-#if defined(HAVE_GLIBC2)
-/* for isnan in Rinlinedfuns.h */
-# define _SVID_SOURCE 1
-#endif
-
-#include <stdlib.h> /* for putenv */
+#include <stdlib.h> /* for setenv or putenv */
 #include <Defn.h> /* for PATH_MAX */
-
+#include <Rinterface.h>
+#include <Fileio.h>
 
 /* remove leading and trailing space */
 static char *rmspace(char *s)
@@ -70,7 +65,7 @@ static char *subterm(char *s)
     } else q = NULL;
     p = getenv(s);
     if(p && strlen(p)) return p; /* variable was set and non-empty */
-    return q ? subterm(q) : "";
+    return q ? subterm(q) : (char *) "";
 }
 
 /* skip along until we find an unmatched right brace */
@@ -110,7 +105,7 @@ static char *findterm(char *s)
 	/* copy over leading part */
 	nans = strlen(ans);
 	strncat(ans, s, p-s); ans[nans + p - s] = '\0';
-	r = alloca(q - p + 2);
+	r = (char *) alloca(q - p + 2);
 	strncpy(r, p, q - p + 1);
 	r[q - p + 1] = '\0';
 	r2 = subterm(r);
@@ -127,10 +122,16 @@ static void Putenv(char *a, char *b)
     char *buf, *value, *p, *q, quote='\0';
     int inquote = 0;
 
+#ifdef HAVE_SETENV
+    buf = (char *) malloc((strlen(b) + 1) * sizeof(char));
+    if(!buf) R_Suicide("allocation failure in reading Renviron");
+    value = buf;
+#else
     buf = (char *) malloc((strlen(a) + strlen(b) + 2) * sizeof(char));
     if(!buf) R_Suicide("allocation failure in reading Renviron");
     strcpy(buf, a); strcat(buf, "=");
     value = buf+strlen(buf);
+#endif
 
     /* now process the value */
     for(p = b, q = value; *p; p++) {
@@ -153,24 +154,32 @@ static void Putenv(char *a, char *b)
 	*q++ = *p;
     }
     *q = '\0';
-#ifdef HAVE_PUTENV
-    putenv(buf);
+#ifdef HAVE_SETENV
+    if(setenv(a, buf, 1))
+	warningcall(R_NilValue, 
+		    _("problem in setting variable '%s' in Renviron"), a);
+    free(buf);
+#elif defined(HAVE_PUTENV)
+    if(putenv(buf))
+	warningcall(R_NilValue, 
+		    _("problem in setting variable '%s' in Renviron"), a);
+    /* no free here: storage remains in use */
 #else
     /* pretty pointless, and was not tested prior to 2.3.0 */
+    free(buf);
 #endif
-    /* no free here: storage remains in use */
 }
 
 
 #define BUF_SIZE 255
 #define MSG_SIZE 2000
-static int process_Renviron(char *filename)
+static int process_Renviron(const char *filename)
 {
     FILE *fp;
     char *s, *p, sm[BUF_SIZE], *lhs, *rhs, msg[MSG_SIZE+50];
     int errs = 0;
 
-    if (!filename || !(fp = fopen(filename, "r"))) return 0;
+    if (!filename || !(fp = R_fopen(filename, "r"))) return 0;
     snprintf(msg, MSG_SIZE+50,
 	     "\n   File %s contains invalid line(s)", filename);
 
@@ -246,7 +255,7 @@ void process_site_Renviron ()
 /* try user Renviron: ./.Renviron, then ~/.Renviron */
 void process_user_Renviron()
 {
-    char *s;
+    const char *s;
 
     if(process_Renviron(".Renviron")) return;
 #ifdef Unix
