@@ -13,8 +13,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -24,7 +24,8 @@
 #include <Defn.h>
 
 
-SEXP attribute_hidden do_mapply(SEXP f, SEXP varyingArgs, SEXP constantArgs, SEXP rho)
+SEXP attribute_hidden 
+do_mapply(SEXP f, SEXP varyingArgs, SEXP constantArgs, SEXP rho)
 {
 
     int i, j, m,nc, *lengths, *counters, named, longest=0;
@@ -60,18 +61,17 @@ SEXP attribute_hidden do_mapply(SEXP f, SEXP varyingArgs, SEXP constantArgs, SEX
     else
 	error(_("argument 'MoreArgs' of 'mapply' is not a list"));
 
-    for(j = m-1; j >= 0;j--) {
-	SET_VECTOR_ELT(mindex,j, allocVector(INTSXP,1));
-	SET_VECTOR_ELT(nindex,j, allocVector(INTSXP,1));
-	INTEGER(VECTOR_ELT(mindex,j))[0] = j+1;
+    for(j = m-1; j >= 0; j--) {
+	SET_VECTOR_ELT(mindex, j, ScalarInteger(j+1));
+	SET_VECTOR_ELT(nindex, j, allocVector(INTSXP, 1));
 
 	PROTECT(tmp1=lang3(R_Bracket2Symbol,
 			   install("dots"),
-			   VECTOR_ELT(mindex,j)));
+			   VECTOR_ELT(mindex, j)));
 
 	PROTECT(tmp2=lang3(R_Bracket2Symbol,
 			   tmp1,
-			   VECTOR_ELT(nindex,j)));
+			   VECTOR_ELT(nindex, j)));
 
 
 	UNPROTECT(3);
