@@ -16,8 +16,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,10 +28,11 @@
 #include <R_ext/Random.h>
 #include <R_ext/Applic.h>	/* for rcont2() */
 #include <Rmath.h>		/* for rxxx functions */
+#include <errno.h>
 
 static void invalid(SEXP call)
 {
-    errorcall(call, _("invalid arguments"));
+    error(_("invalid arguments"));
 }
 
 static Rboolean random1(double (*f) (double), double *a, int na, double *x, int n)
@@ -96,7 +97,7 @@ SEXP attribute_hidden do_random1(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("internal error in do_random1"));
 	}
 	if (naflag)
-	    warningcall(call, _("NAs produced"));
+	    warning(_("NAs produced"));
 
 	PutRNGstate();
 	UNPROTECT(1);
@@ -178,7 +179,7 @@ SEXP attribute_hidden do_random2(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("internal error in do_random2"));
 	}
 	if (naflag)
-	    warningcall(call, _("NAs produced"));
+	    warning(_("NAs produced"));
 
 	PutRNGstate();
 	UNPROTECT(2);
@@ -256,7 +257,7 @@ SEXP attribute_hidden do_random3(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("internal error in do_random3"));
 	}
 	if (naflag)
-	    warningcall(call, _("NAs produced"));
+	    warning(_("NAs produced"));
 
 	PutRNGstate();
 	UNPROTECT(3);
@@ -310,6 +311,8 @@ static Rboolean Walker_warn = FALSE;
 /* A  version using Walker's alias method, based on Alg 3.13B in
    Ripley (1987).
  */
+
+#define SMALL 10000
 static void 
 walker_ProbSampleReplace(int n, double *p, int *a, int nans, int *ans) 
 {
@@ -328,7 +331,7 @@ walker_ProbSampleReplace(int n, double *p, int *a, int nans, int *ans)
        and L ... H[n-1] label those >= 1.
        By rounding error we could have q[i] < 1. or > 1. for all entries.
      */
-    if(n <= 10000) {
+    if(n <= SMALL) {
 	/* might do this repeatedly, so speed matters */
 	HL = (int *)alloca(n * sizeof(int));
 	q = (double *) alloca(n * sizeof(double));
@@ -361,7 +364,7 @@ walker_ProbSampleReplace(int n, double *p, int *a, int nans, int *ans)
 	k = (int) rU;
 	ans[i] = (rU < q[k]) ? k+1 : a[k]+1;
     }
-    if(n > 100000) {
+    if(n > SMALL) {
 	Free(HL);
 	Free(q);
     }
@@ -434,16 +437,16 @@ static void FixupProb(SEXP call, double *p, int n, int k, int replace)
     sum = 0.;
     for (i = 0; i < n; i++) {
 	if (!R_FINITE(p[i]))
-	    errorcall(call, _("NA in probability vector"));
+	    error(_("NA in probability vector"));
 	if (p[i] < 0)
-	    errorcall(call, _("non-positive probability"));
+	    error(_("non-positive probability"));
 	if (p[i] > 0) {
 	    npos++;
 	    sum += p[i];
 	}
     }
     if (npos == 0 || (!replace && k > npos))
-	errorcall(call, _("too few positive probabilities"));
+	error(_("too few positive probabilities"));
     for (i = 0; i < n; i++)
 	p[i] /= sum;
 }
@@ -462,17 +465,17 @@ SEXP attribute_hidden do_sample(SEXP call, SEXP op, SEXP args, SEXP rho)
     k = asInteger(CAR(args)); args = CDR(args);
     sreplace = CAR(args); args = CDR(args);
     if( length(sreplace) != 1 )
-         errorcall(call, _("invalid '%s' argument"), "replace");
+         error(_("invalid '%s' argument"), "replace");
     replace = asLogical(sreplace);
     prob = CAR(args);
     if (replace == NA_LOGICAL)
-	errorcall(call, _("invalid '%s' argument"), "replace");
+	error(_("invalid '%s' argument"), "replace");
     if (n == NA_INTEGER || n < 1)
-	errorcall(call, _("invalid '%s' argument"), "x");
+	error(_("invalid '%s' argument"), "x");
     if (k == NA_INTEGER || k < 0)
-	errorcall(call, _("invalid '%s' argument"), "size");
+	error(_("invalid '%s' argument"), "size");
     if (!replace && k > n)
-	errorcall(call, _("cannot take a sample larger than the population\n when 'replace = FALSE'"));
+	error(_("cannot take a sample larger than the population when 'replace = FALSE'"));
     GetRNGstate();
     PROTECT(y = allocVector(INTSXP, k));
     if (!isNull(prob)) {
@@ -481,7 +484,7 @@ SEXP attribute_hidden do_sample(SEXP call, SEXP op, SEXP args, SEXP rho)
 	PROTECT(prob);
 	p = REAL(prob);
 	if (length(prob) != n)
-	    errorcall(call, _("incorrect number of probabilities"));
+	    error(_("incorrect number of probabilities"));
 	FixupProb(call, p, n, k, replace);
 	PROTECT(x = allocVector(INTSXP, n));
 	if (replace) {
@@ -516,9 +519,9 @@ SEXP attribute_hidden do_rmultinom(SEXP call, SEXP op, SEXP args, SEXP rho)
     n	 = asInteger(CAR(args)); args = CDR(args);/* n= #{samples} */
     size = asInteger(CAR(args)); args = CDR(args);/* X ~ Multi(size, prob) */
     if (n == NA_INTEGER || n < 0)
-	errorcall(call, _("invalid first argument 'n'"));
+	error(_("invalid first argument 'n'"));
     if (size == NA_INTEGER || size < 0)
-	errorcall(call, _("invalid second argument 'size'"));
+	error(_("invalid second argument 'size'"));
     prob = CAR(args);
     prob = coerceVector(prob, REALSXP);
     k = length(prob);/* k = #{components or classes} = X-vector length */
