@@ -15,12 +15,12 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is either ASCII or handled as a whole or as 
-   a leading portion for partial matching 
+/* <UTF8> char here is either ASCII or handled as a whole or as
+   a leading portion for partial matching
 */
 
 
@@ -28,11 +28,6 @@
 #include <config.h>
 #endif
 
-#if defined(HAVE_GLIBC2)
-/* for isnan in Rinlinedfuns.h */
-# define _SVID_SOURCE 1
-#endif
-
 #include <Defn.h>
 
 #define NIL -1
@@ -52,15 +47,15 @@ struct _HashData {
 };
 
 
-/* 
+/*
    Integer keys are hashed via a random number generator
    based on Knuth's recommendations.  The high order K bits
    are used as the hash code.
-   
+
    NB: lots of this code relies on M being a power of two and
    on silent integer overflow mod 2^32.  It also relies on M < 31.
 
-   <FIXME>  Integer keys are wasteful for logical and raw vectors, 
+   <FIXME>  Integer keys are wasteful for logical and raw vectors,
    but the tables are small in that case.
 */
 
@@ -140,7 +135,7 @@ static int chash(SEXP x, int indx, HashData *d)
 static int shash(SEXP x, int indx, HashData *d)
 {
     unsigned int k;
-    char *p = translateChar(STRING_ELT(x, indx));
+    const char *p = translateChar(STRING_ELT(x, indx));
     k = 0;
     while (*p++)
 	    k = 11 * k + *p; /* was 8 but 11 isn't a power of 2 */
@@ -186,14 +181,14 @@ static int cequal(SEXP x, int i, SEXP y, int j)
 
 static int sequal(SEXP x, int i, SEXP y, int j)
 {
-    /* Two strings which have the same address must be the same, 
+    /* Two strings which have the same address must be the same,
        so avoid looking at the contents */
     if (STRING_ELT(x, i) == STRING_ELT(y, j)) return 1;
     /* Then if either is NA the other cannot be */
-    if (STRING_ELT(x, i) == NA_STRING || STRING_ELT(y, j) == NA_STRING) 
+    if (STRING_ELT(x, i) == NA_STRING || STRING_ELT(y, j) == NA_STRING)
 	return 0;
     /* Finally look at the contents if necessary */
-    return !strcmp(translateChar(STRING_ELT(x, i)), 
+    return !strcmp(translateChar(STRING_ELT(x, i)),
 		   translateChar(STRING_ELT(y, j)));
 }
 
@@ -212,7 +207,7 @@ static int vhash(SEXP x, int indx, HashData *d)
     int i;
     unsigned int key;
     SEXP _this = VECTOR_ELT(x, indx);
-    
+
     key = OBJECT(_this) + 2*TYPEOF(_this) + 100*length(_this);
     /* maybe we should also look at attributes, but that slows us down */
     switch (TYPEOF(_this)) {
@@ -357,14 +352,14 @@ static int isDuplicated(SEXP x, int indx, HashData *d)
     return 0;
 }
 
-SEXP duplicated(SEXP x)
+SEXP duplicated(SEXP x, Rboolean from_last)
 {
     SEXP ans;
     int *h, *v;
     int i, n;
     HashData data;
 
-    if (!isVector(x)) 
+    if (!isVector(x))
 	error(_("'duplicated' applies only to vectors"));
 
     n = LENGTH(x);
@@ -378,9 +373,13 @@ SEXP duplicated(SEXP x)
     for (i = 0; i < data.M; i++)
 	h[i] = NIL;
 
-    for (i = 0; i < n; i++)
-	v[i] = isDuplicated(x, i, &data);
-
+    if(from_last) {
+	for (i = n-1; i >= 0; i--)
+	    v[i] = isDuplicated(x, i, &data);
+    } else {
+	for (i = 0; i < n; i++)
+	    v[i] = isDuplicated(x, i, &data);
+    }
     return ans;
 }
 
@@ -404,7 +403,7 @@ SEXP attribute_hidden do_duplicated(SEXP call, SEXP op, SEXP args, SEXP env)
 	      (PRIMVAL(op) == 0 ? "duplicated" : "unique"));
     }
 
-    dup = duplicated(x);
+    dup = duplicated(x, asLogical(CADR(args)));
     if (PRIMVAL(op) == 0) /* "duplicated()" : */
 	return dup;
     /*	ELSE
@@ -570,67 +569,115 @@ SEXP match(SEXP itable, SEXP ix, int nmatch)
 SEXP attribute_hidden do_pmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans, input, target;
-    int i, j, k, mtch, n_input, n_target, mtch_count, temp, dups_ok;
-    int * used;
-    char *vmax;
+    int i, j,  mtch, n_input, n_target, mtch_count, temp, dups_ok, no_match;
+    int nexact = 0;
+    int *used = NULL, *ians;
+    const char **in, **tar;
+    Rboolean no_dups;
 
     checkArity(op, args);
-    vmax = vmaxget();
     input = CAR(args);
     n_input = LENGTH(input);
     target = CADR(args);
     n_target = LENGTH(target);
-    dups_ok = asLogical(CADDR(args));
+    no_match = asInteger(CADDR(args));
+    dups_ok = asLogical(CADDDR(args));
     if (dups_ok == NA_LOGICAL)
-	errorcall(call, _("invalid '%s' argument"), "duplicates.ok");
+	error(_("invalid '%s' argument"), "duplicates.ok");
+    no_dups = !dups_ok;
 
     if (!isString(input) || !isString(target))
-	errorcall(call, _("argument is not of mode character"));
+	error(_("argument is not of mode character"));
 
-    used = (int *) R_alloc(n_target, sizeof(int));
-    for (j = 0; j < n_target; j++) used[j] = 0;
-    ans = allocVector(INTSXP, n_input);
-    for (i = 0; i < n_input; i++) INTEGER(ans)[i] = 0;
+    if(no_dups) {
+	used = (int *) R_alloc(n_target, sizeof(int));
+	for (j = 0; j < n_target; j++) used[j] = 0;
+    }
+
+    in = (const char **) R_alloc(n_input, sizeof(char *));
+    tar = (const char **) R_alloc(n_target, sizeof(char *));
+    PROTECT(ans = allocVector(INTSXP, n_input));
+    ians = INTEGER(ans);
+    for (i = 0; i < n_input; i++) {
+	in[i] = translateChar(STRING_ELT(input, i));
+	ians[i] = 0;
+    }
+    for (j = 0; j < n_target; j++)
+	tar[j] = translateChar(STRING_ELT(target, j));
 
     /* First pass, exact matching */
-    for (i = 0; i < n_input; i++) {
-	char *ss = translateChar(STRING_ELT(input, i));
-	temp = strlen(ss);
-	if (temp == 0) continue;
-	for (j = 0; j < n_target; j++) {
-	    if (!dups_ok && used[j]) continue;
-	    k = strcmp(ss, translateChar(STRING_ELT(target, j)));
-	    if (k == 0) {
-		used[j] = 1;
-		INTEGER(ans)[i] = j + 1;
-		break;
+    if(no_dups) {
+	for (i = 0; i < n_input; i++) {
+	    const char *ss = in[i];
+	    if (strlen(ss) == 0) continue;
+	    for (j = 0; j < n_target; j++) {
+		if (used[j]) continue;
+		if (strcmp(ss, tar[j]) == 0) {
+		    if(no_dups) used[j] = 1;
+		    ians[i] = j + 1;
+		    nexact++;
+		    break;
+		}
 	    }
 	}
-    }
-    /* Second pass, partial matching */
-    for (i = 0; i < n_input; i++) {
-	char *ss;
-	if (INTEGER(ans)[i]) continue;
-	ss = translateChar(STRING_ELT(input, i));
-	temp = strlen(ss);
-	if (temp == 0) continue;
-	mtch = 0;
-	mtch_count = 0;
-	for (j = 0; j < n_target; j++) {
-	    if (!dups_ok && used[j]) continue;
-	    k = strncmp(ss, translateChar(STRING_ELT(target, j)), temp);
-	    if (k == 0) {
-		mtch = j + 1;
-		mtch_count++;
+    } else {
+	/* only worth hashing if enough lookups will be done:
+	   since the tradeoff involves memory as well as time
+	   it is not really possible to optimize there.
+	 */
+	if(n_target > 100 && 10*n_input > n_target) {
+	    HashData data;
+	    HashTableSetup(target, &data);
+	    data.nomatch = 0;
+	    DoHashing(target, &data);
+	    for (i = 0; i < n_input; i++) {
+		/* we don't want to lookup "" */
+		if (strlen(in[i]) == 0) continue;
+		ians[i] = Lookup(target, input, i, &data);
+		if(ians[i]) nexact++;
+	    }
+	} else {
+	    for (i = 0; i < n_input; i++) {
+		const char *ss = in[i];
+		if (strlen(ss) == 0) continue;
+		for (j = 0; j < n_target; j++)
+		    if (strcmp(ss, tar[j]) == 0) {
+			ians[i] = j + 1;
+			nexact++;
+			break;
+		    }
 	    }
-	}
-	if (mtch > 0 && mtch_count == 1) {
-	    used[mtch - 1] = 1;
-	    INTEGER(ans)[i] = mtch;
 	}
     }
 
-    vmaxset(vmax);
+    if(nexact < n_input) {
+	/* Second pass, partial matching */
+	for (i = 0; i < n_input; i++) {
+	    const char *ss;
+	    if (ians[i]) continue;
+	    ss = in[i];
+	    temp = strlen(ss);
+	    if (temp == 0) continue;
+	    mtch = 0;
+	    mtch_count = 0;
+	    for (j = 0; j < n_target; j++) {
+		if (no_dups && used[j]) continue;
+		if (strncmp(ss, tar[j], temp) == 0) {
+		    mtch = j + 1;
+		    mtch_count++;
+		}
+	    }
+	    if (mtch > 0 && mtch_count == 1) {
+		if(no_dups) used[mtch - 1] = 1;
+		ians[i] = mtch;
+	    }
+	}
+	/* Third pass, set no matches */
+	for (i = 0; i < n_input; i++)
+	    if(ians[i] == 0) ians[i] = no_match;
+
+    }
+    UNPROTECT(1);
     return ans;
 }
 
@@ -642,8 +689,8 @@ SEXP attribute_hidden do_charmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP ans, input, target;
     Rboolean perfect;
-    int i, j, k, imatch, n_input, n_target, temp;
-    char *ss, *st;
+    int i, j, k, imatch, n_input, n_target, temp, no_match, *ians;
+    const char *ss, *st;
 
     checkArity(op, args);
 
@@ -653,9 +700,11 @@ SEXP attribute_hidden do_charmatch(SEXP call, SEXP op, SEXP args, SEXP env)
     n_target = LENGTH(target);
 
     if (!isString(input) || !isString(target))
-	errorcall(call, _("argument is not of mode character"));
+	error(_("argument is not of mode character"));
+    no_match = asInteger(CADDR(args));
 
     PROTECT(ans = allocVector(INTSXP, n_input));
+    ians = INTEGER(ans);
 
     for (i = 0; i < n_input; i++) {
 	ss = translateChar(STRING_ELT(input, i));
@@ -682,7 +731,7 @@ SEXP attribute_hidden do_charmatch(SEXP call, SEXP op, SEXP args, SEXP env)
 		}
 	    }
 	}
-	INTEGER(ans)[i] = imatch;
+	INTEGER(ans)[i] = (imatch == NA_INTEGER) ? no_match : imatch;
     }
     UNPROTECT(1);
     return ans;
@@ -781,7 +830,7 @@ SEXP attribute_hidden do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
 	funcall = VECTOR_ELT(funcall, 0);
 
     if (TYPEOF(funcall) != LANGSXP)
-	errorcall(call, _("invalid '%s' argument"), "call");
+	error(_("invalid '%s' argument"), "call");
 
     /* Get the function definition */
     sysp = R_GlobalContext->sysparent;
@@ -795,9 +844,11 @@ SEXP attribute_hidden do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
 		break;
 	    cptr = cptr->nextcontext;
 	}
-	if ( cptr == NULL )
+	if ( cptr == NULL ) {
 	    sysp = R_GlobalEnv;
-	else
+	    errorcall(R_NilValue,
+		      "match.call() was called from outside a function");
+	} else
 	    sysp = cptr->sysparent;
 	if (cptr != NULL)
 	    /* Changed to use the function from which match.call was
@@ -830,19 +881,23 @@ SEXP attribute_hidden do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
 	    PROTECT(b = findFun(CAR(funcall), sysp));
 	else
 	    PROTECT(b = eval(CAR(funcall), sysp));
+
+	if (TYPEOF(b) != CLOSXP)
+	    error(_("unable to find a closure from within which 'match.call' was called"));
+
+    }
+    else {
+	/* It must be a closure! */
+	PROTECT(b = CAR(args));
+	if (TYPEOF(b) != CLOSXP)
+	    error(_("invalid '%s' argument"), "definition");
     }
-    else PROTECT(b = CAR(args));
-
-    /* It must be a closure! */
-
-    if (TYPEOF(b) != CLOSXP)
-	errorcall(call, _("invalid '%s' argument"), "definition");
 
     /* Do we expand ... ? */
 
     expdots = asLogical(CAR(CDDR(args)));
     if (expdots == NA_LOGICAL)
-	errorcall(call, _("invalid '%s' argument"), "expand.dots");
+	error(_("invalid '%s' argument"), "expand.dots");
 
     /* Get the formals and match the actual args */
 
@@ -893,7 +948,7 @@ SEXP attribute_hidden do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
 	    }
 	}
     }
-    rlist = matchArgs(formals, actuals);
+    rlist = matchArgs(formals, actuals, call);
 
     /* Attach the argument names as tags */
 
@@ -932,7 +987,7 @@ SEXP attribute_hidden do_matchcall(SEXP call, SEXP op, SEXP args, SEXP env)
 #  define ZERODBL(X,N,I) for(I=0;I<N;I++) REAL(X)[I]=0
 #endif
 
-SEXP attribute_hidden 
+SEXP attribute_hidden
 Rrowsum_matrix(SEXP x, SEXP ncol, SEXP g, SEXP uniqueg, SEXP snarm)
 {
     SEXP matches,ans;
@@ -961,7 +1016,7 @@ Rrowsum_matrix(SEXP x, SEXP ncol, SEXP g, SEXP uniqueg, SEXP snarm)
 	for(i = 0; i < p; i++) {
 	    for(j = 0; j < n; j++)
 		if(!narm || !ISNAN(REAL(x)[j+offset]))
-		    REAL(ans)[INTEGER(matches)[j]-1+offsetg] 
+		    REAL(ans)[INTEGER(matches)[j]-1+offsetg]
 			+= REAL(x)[j+offset];
 	    offset += n;
 	    offsetg += ng;
@@ -973,11 +1028,11 @@ Rrowsum_matrix(SEXP x, SEXP ncol, SEXP g, SEXP uniqueg, SEXP snarm)
 	    for(j = 0; j < n; j++) {
 		if (INTEGER(x)[j+offset] == NA_INTEGER) {
 		    if(!narm)
-			INTEGER(ans)[INTEGER(matches)[j]-1+offsetg] 
+			INTEGER(ans)[INTEGER(matches)[j]-1+offsetg]
 			    = NA_INTEGER;
-		} else if (INTEGER(ans)[INTEGER(matches)[j]-1+offsetg] 
+		} else if (INTEGER(ans)[INTEGER(matches)[j]-1+offsetg]
 			 != NA_INTEGER)
-		    INTEGER(ans)[INTEGER(matches)[j]-1+offsetg] 
+		    INTEGER(ans)[INTEGER(matches)[j]-1+offsetg]
 			+= INTEGER(x)[j+offset];
 	    }
 	    offset += n;
@@ -1094,18 +1149,16 @@ SEXP attribute_hidden do_makeunique(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP names, sep, ans, dup, newx;
     int i, n, cnt, len, maxlen = 0, *cnts, dp;
     HashData data;
-    char *csep, *buf, *ss;
-    char *vmax;
+    const char *csep, *ss; char *buf;
 
     checkArity(op, args);
-    vmax = vmaxget();
     names = CAR(args);
     if(!isString(names))
-	errorcall(call, _("'names' must be a character vector"));
+	error(_("'names' must be a character vector"));
     n = LENGTH(names);
     sep = CADR(args);
     if(!isString(sep) || LENGTH(sep) != 1)
-	errorcall(call, _("'sep' must be a character string"));
+	error(_("'sep' must be a character string"));
     csep = translateChar(STRING_ELT(sep, 0));
     PROTECT(ans = allocVector(STRSXP, n));
     for(i = 0; i < n; i++) {
@@ -1146,12 +1199,11 @@ SEXP attribute_hidden do_makeunique(SEXP call, SEXP op, SEXP args, SEXP env)
 	UNPROTECT(3);
     }
     UNPROTECT(1);
-    vmaxset(vmax);
     return ans;
 }
 
-/* Use hashing to improve object.size. Here we want equal CHARSXPs, 
-   not equal contents.  This only uses the bottom 32 bits of the pointer, 
+/* Use hashing to improve object.size. Here we want equal CHARSXPs,
+   not equal contents.  This only uses the bottom 32 bits of the pointer,
    but for now that's almost certainly OK */
 
 static int cshash(SEXP x, int indx, HashData *d)
