@@ -73,7 +73,22 @@ static Rboolean url_open(Rconnection con)
     case HTTPSsh:
 #endif
     case HTTPsh:
-	ctxt = in_R_HTTPOpen(url, NULL, 0);
+    {
+	SEXP sheaders, agentFun;
+	const char *headers;
+#ifdef USE_WININET
+	PROTECT(agentFun = lang2(install("makeUserAgent"), ScalarLogical(0)));
+#else
+	PROTECT(agentFun = lang1(install("makeUserAgent")));
+#endif
+	PROTECT(sheaders = eval(agentFun, R_FindNamespace(mkString("utils"))));
+	
+	if(TYPEOF(sheaders) == NILSXP)
+	    headers = NULL;
+	else
+	    headers = CHAR(STRING_ELT(sheaders, 0));
+	ctxt = in_R_HTTPOpen(url, headers, 0);
+	UNPROTECT(2);
 	if(ctxt == NULL) {
 	  /* if we call error() we get a connection leak*/
 	  /* so do_url has to raise the error*/
@@ -81,6 +96,7 @@ static Rboolean url_open(Rconnection con)
 	    return FALSE;
 	}
 	((Rurlconn)(con->private))->ctxt = ctxt;
+    }
 	break;
     case FTPsh:
 	ctxt = in_R_FTPOpen(url);
